@@ -1,5 +1,5 @@
 /*
- * $Id: cap_alloc.c,v 1.1.1.1.2.1 1999/04/23 06:16:31 morgan Exp $
+ * $Id: cap_alloc.c,v 1.1.1.1.2.2 1999/05/01 23:33:32 morgan Exp $
  *
  * Copyright (c) 1997-8 Andrew G Morgan <morgan@linux.kernel.org>
  *
@@ -12,7 +12,61 @@
 #include "libcap.h"
 
 /*
- * This function duplicates an internal capability set (x3) with
+ * Obtain a blank set of capabilities
+ */
+
+cap_t cap_init(void)
+{
+    __u32 *raw_data;
+    cap_t result;
+
+    raw_data = malloc( sizeof(__u32) + sizeof(*result) );
+
+    if (raw_data == NULL) {
+	_cap_debug("out of memory");
+	errno = ENOMEM;
+	return NULL;
+    }
+
+    *raw_data = CAP_T_MAGIC;
+    result = (cap_t) (raw_data + 1);
+    memset(result, 0, sizeof(*result));
+
+    _libcap_establish_api();
+    result->features = _libcap_kernel_features;
+    result->head.version = _libcap_kernel_version;
+
+    return result;
+}
+
+/*
+ * This is an internal library function to duplicate a string and
+ * tag the result as something cap_free can handle.
+ */
+
+char *_libcap_strdup(const char *old)
+{
+    __u32 *raw_data;
+
+    if (old == NULL) {
+	errno = EINVAL;
+	return NULL;
+    }
+
+    raw_data = malloc( sizeof(__u32) + strlen(old) + 1 );
+    if (raw_data == NULL) {
+	errno = ENOMEM;
+	return NULL;
+    }
+
+    *(raw_data++) = CAP_S_MAGIC;
+    strcpy((char *) raw_data, old);
+
+    return ((char *) raw_data);
+}
+
+/*
+ * This function duplicates an internal capability set with
  * malloc()'d memory. It is the responsibility of the user to call
  * cap_free() to liberate it.
  */
@@ -27,10 +81,9 @@ cap_t cap_dup(cap_t cap_d)
 	return NULL;
     }
 
-    result = (cap_t) malloc( sizeof(*cap_d) );
+    result = cap_init();
     if (result == NULL) {
 	_cap_debug("out of memory");
-	errno = ENOMEM;
 	return NULL;
     }
 
@@ -44,45 +97,36 @@ cap_t cap_dup(cap_t cap_d)
  * Scrub and then liberate an internal capability set.
  */
 
-int cap_free(cap_t *cap_d_p)
+int cap_free(void *data_p)
 {
-    if ( cap_d_p && good_cap_t(*cap_d_p) ) {
-	memset(*cap_d_p, 0, sizeof(**cap_d_p));
-	free(*cap_d_p);
-	*cap_d_p = NULL;
 
+    if ( good_cap_t(data_p) ) {
+	data_p = -1 + (__u32 *) data_p;
+	memset(data_p, 0, sizeof(__u32) + sizeof(struct _cap_struct));
+	free(data_p);
+	data_p = NULL;
 	return 0;
-    } else {
-	_cap_debug("no capability to liberate");
-	errno = EINVAL;
-	return -1;
     }
-}
 
-/*
- * Obtain a blank set of capabilities
- */
-
-cap_t cap_init(void)
-{
-    cap_t result = (cap_t) calloc( 1, sizeof(*result) );
-
-    if (result) {
-
-	result->magic = CAP_T_MAGIC;
-
-	_libcap_establish_api();
-	result->features = _libcap_kernel_features;
-	result->head.version = _libcap_kernel_version;
-
-    } else {
-	errno = ENOMEM;
+    if ( good_cap_string(data_p) ) {
+	int length = strlen(data_p) + sizeof(__u32);
+	data_p = -1 + (__u32 *) data_p;
+	memset(data_p, 0, length);
+	free(data_p);
+	data_p = NULL;
+	return 0;
     }
-    return result;
+
+    _cap_debug("don't recognize what we're supposed to liberate");
+    errno = EINVAL;
+    return -1;
 }
 
 /*
  * $Log: cap_alloc.c,v $
+ * Revision 1.1.1.1.2.2  1999/05/01 23:33:32  morgan
+ * made cap_free responsible for liberating strings returned by cap_to_text
+ *
  * Revision 1.1.1.1.2.1  1999/04/23 06:16:31  morgan
  * added support for revised kernel interface (with file cap support)
  *
