@@ -1,5 +1,5 @@
 /*
- * $Id: cap_text.c,v 1.3 1999/11/18 06:03:26 morgan Exp $
+ * $Id: cap_text.c,v 1.2.2.1 1999/05/01 23:33:32 morgan Exp $
  *
  * Copyright (c) 1997-8 Andrew G Morgan <morgan@linux.kernel.org>
  * Copyright (c) 1997 Andrew Main <zefram@dcs.warwick.ac.uk>
@@ -16,6 +16,8 @@
 #include <ctype.h>
 #include <stdio.h>
 
+char *strdup(const char *s);
+
 /* Maximum output text length (16 per cap) */
 #define CAP_TEXT_SIZE    (16*__CAP_BITS)
 
@@ -238,7 +240,7 @@ static int getstateflags(cap_t caps, int capno)
 
 char *cap_to_text(cap_t caps, ssize_t *length_p)
 {
-    static char buf[CAP_TEXT_SIZE+CAP_TEXT_BUFFER_ZONE];
+    char *buf, *cap_string = NULL;
     char *p;
     int histo[8] = {0};
     int m, n, t;
@@ -249,6 +251,12 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 	return NULL;
     }
 
+    buf = calloc(1, CAP_TEXT_SIZE+CAP_TEXT_BUFFER_ZONE);
+    if (buf == NULL) {
+	errno = ENOMEM;
+	return NULL;
+    }
+
     _cap_debugcap("e = ", &caps->set.effective);
     _cap_debugcap("i = ", &caps->set.inheritable);
     _cap_debugcap("p = ", &caps->set.permitted);
@@ -277,7 +285,7 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 			p += sprintf(p, "%d,", n);
 		    if (p - buf > CAP_TEXT_SIZE) {
 			errno = ERANGE;
-			return NULL;
+			goto drop_buf;
 		    }
 		}
 	    p--;
@@ -295,7 +303,7 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 			     (n & LIBCAP_PER) ? "p" : "");
 	    if (p - buf > CAP_TEXT_SIZE) {
 		errno = ERANGE;
-		return NULL;
+		goto drop_buf;
 	    }
 	}
 
@@ -304,13 +312,17 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 	*length_p = p - buf;
     }
 
-    return (_libcap_strdup(buf));
+    cap_string = _libcap_strdup(buf);
+
+drop_buf:
+    memset(buf, 0, strlen(buf));
+    return cap_string;
 }
 
 /*
  * $Log: cap_text.c,v $
- * Revision 1.3  1999/11/18 06:03:26  morgan
- * fixed cap_free to work as indicated in manuals
+ * Revision 1.2.2.1  1999/05/01 23:33:32  morgan
+ * made cap_free responsible for liberating strings returned by cap_to_text
  *
  * Revision 1.2  1999/04/17 23:25:09  morgan
  * fixes from peeterj
