@@ -1,5 +1,5 @@
 /*
- * $Id: libcap.h,v 1.2.2.1 1999/04/23 06:16:31 morgan Exp $
+ * $Id: libcap.h,v 1.2.2.2 1999/05/01 23:33:32 morgan Exp $
  *
  * Copyright (c) 1997 Andrew G Morgan <morgan@linux.kernel.org>
  *
@@ -39,14 +39,24 @@
  * to processes.
  */
 
-#define CAP_T_MAGIC 0xCA90D1
 struct _cap_struct {
-    int magic;
     unsigned features;
     struct __user_cap_header_struct head;
     struct __user_cap_data_struct set;
 };
 
+/*
+ * Some magic cookies identifying capability objects that this library
+ * maintains.
+ */
+
+/* this is stored immediately preceeding the capability structure */
+#define CAP_T_MAGIC (0xCA90D1 ^ _LINUX_CAPABILITY_VERSION)
+
+/* this is stored immediately preceeding a string produced by
+   cap_to_text */
+#define CAP_S_MAGIC (0xCA95DA ^ _LINUX_CAPABILITY_VERSION)
+
 /*
  * 'feature' set of methods supported by kernel
  */
@@ -85,7 +95,9 @@ typedef struct {
  * Private definitions for internal use by the library.
  */
 
-#define good_cap_t(c)      ((c) && (c)->magic == CAP_T_MAGIC)
+#define __libcap_check_magic(c,magic) ((c) && *(-1+(__u32 *)(c)) == (magic))
+#define good_cap_t(c)        __libcap_check_magic(c, CAP_T_MAGIC)
+#define good_cap_string(c)   __libcap_check_magic(c, CAP_S_MAGIC)
 
 /*
  * library debugging
@@ -120,6 +132,7 @@ extern int capget(cap_user_header_t header, const cap_user_data_t data);
 extern int capgetp(pid_t pid, cap_t cap_d);
 extern int capsetp(pid_t pid, cap_t cap_d);
 
+extern char *_libcap_strdup(const char *);
 extern void _libcap_establish_api(void);
 extern unsigned int _libcap_kernel_version;
 extern unsigned int _libcap_kernel_features;
@@ -128,6 +141,9 @@ extern unsigned int _libcap_kernel_features;
 
 /*
  * $Log: libcap.h,v $
+ * Revision 1.2.2.2  1999/05/01 23:33:32  morgan
+ * made cap_free responsible for liberating strings returned by cap_to_text
+ *
  * Revision 1.2.2.1  1999/04/23 06:16:31  morgan
  * added support for revised kernel interface (with file cap support)
  *
