@@ -1,5 +1,5 @@
 /*
- * $Id: getcap.c,v 1.1.2.1 1999/04/23 05:00:31 morgan Exp $
+ * $Id: getcap.c,v 1.1.2.2 1999/05/01 23:26:31 morgan Exp $
  *
  * Copyright (c) 1997 Andrew G. Morgan  <morgan@ftp.kernel.org>
  *
@@ -44,7 +44,7 @@ int main(int argc, char **argv)
 
 	result = cap_to_text(cap_d, &length);
 
-	fprintf(stderr, "Capabilities for `%s':\n%s\n", *argv, result);
+	fprintf(stderr, "Capabilities for `%s': %s\n", *argv, result);
     }
 
     return 0;
@@ -52,6 +52,11 @@ int main(int argc, char **argv)
 
 /*
  * $Log: getcap.c,v $
+ * Revision 1.1.2.2  1999/05/01 23:26:31  morgan
+ * added list of things to ignore in this directory
+ * altered command line syntax of sucap and also made it work (sometimes)
+ * getcap output is now one line per file
+ *
  * Revision 1.1.2.1  1999/04/23 05:00:31  morgan
  * resurecting from the dead for kernels with fcap support
  *
